/**
 * LICENSE 
 * Bonita
 * Copyright (C) 2005 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.proed.chi.test;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jdom.Document;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.ow2.bonita.util.Misc;
import org.ow2.proed.Trace;
import org.ow2.proed.Utils;
import org.ow2.proed.chi.mainFrame.MainFrame;
import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.BlockActivity;
import org.ow2.proed.graphModel.Participant;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.RouteActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.Transition;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.ProcessActivity.ExecutionMode;
import org.ow2.proed.graphModel.exception.ParticipantTypeException;
import org.ow2.proed.graphModel.exception.TransitionException;
import org.ow2.proed.resources.PropertiesManager;
import org.ow2.proed.xpdl.XPDLProEdProject;
import org.ow2.proed.xpdl.XPDLProperty;

/**
 * XPDLtest.java Created on 21 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: XPDLtest.java 4444 2009-02-27 08:36:36Z souillac $
 *          <p>
 */
public class XPDLtest extends TestCase {

  private ProEdProject project;

  private XPDLProEdProject xpdlProject;

  private final String fileName = "TestJUnit.xpdl";

  // trace for error
  private Trace trace;

  // process activities to test
  private AtomicActivity a1;

  private BlockActivity a2;

  private RouteActivity a3;

  private Subflow s;

  // transitions to test
  private Transition t1;

  private Transition t2;

  private Participant participant;

  private final PropertiesManager propManager = new PropertiesManager();

  public XPDLtest(final String name) {
    super(name);
  }

  @Override
  protected void setUp() {

    this.project = new ProEdProject("Project1", false);
    final WorkflowProcesses workflows = new WorkflowProcesses();
    final WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
    this.project.setMyProcesses(workflows);
    workflows.add(mainWorkflow);

    // set up all process activities
    this.a1 = new AtomicActivity("Activity 1", null, null, null, ExecutionMode.Synchronous, mainWorkflow, null, null);
    this.a2 = new BlockActivity("Activity 2", null, null, null, ExecutionMode.Synchronous, this.project.getMyProcesses().getMainWorkflow(), null);
    this.a3 = new RouteActivity("Activity 3", null, null, null, ExecutionMode.Synchronous, mainWorkflow, null);
    this.s = new Subflow("subflow 1", null, null, ExecutionMode.Synchronous, mainWorkflow, null);

    // set up all transitions
    try {
      // set up bad transitions
      this.t1 = new Transition(null, this.a2, mainWorkflow);
      Assert.assertNull(this.t1);

      this.t1 = new Transition(null, null, mainWorkflow);
      Assert.assertNull(this.t1);

      this.t1 = new Transition(this.a1, null, mainWorkflow);
      Assert.assertNull(this.t1);

    } catch (final TransitionException e1) {
      System.out.println(e1.getMessage());
    }
    try {
      // set up good transitions
      this.t1 = new Transition(this.a1, this.a2, mainWorkflow);
      Assert.assertNotNull(this.t1);
      this.t2 = new Transition(this.a2, this.a3, mainWorkflow);
      Assert.assertNotNull(this.t2);
    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }

    try {
      // set up bad participant
      this.participant = new Participant("Manue", null);
      Assert.assertNull(this.participant);
      this.participant = new Participant("Manue", "manue", mainWorkflow);
      Assert.assertNull(this.participant);

    } catch (final ParticipantTypeException e) {

      System.out.println(e.getMessage());
    }

    try {
      // set up good participant
      this.participant = new Participant("Manue", Participant.CST_HUMAN, mainWorkflow);
      Assert.assertNotNull(this.participant);
    } catch (final ParticipantTypeException e) {
      System.out.println(e.getMessage());
    }

    this.trace = new Trace(true, true);
  }

  public void testAddingOfActivities() {
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a1);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a2);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a3);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.s);

    Assert.assertNotNull(this.project.getMyProcesses().getMainWorkflow().getProcessActivities());
    Assert.assertEquals(this.project.getMyProcesses().getMainWorkflow().getProcessActivities().size(), 4);

  }

  public void testAddingOfTransition() {
    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t1.getSource(), this.t1.getTarget());
    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t2.getSource(), this.t2.getTarget());

    Assert.assertNotNull(this.project.getMyProcesses().getMainWorkflow().getTransitions());
    Assert.assertEquals(this.project.getMyProcesses().getMainWorkflow().getTransitions().size(), 2);

  }

  public void testGenerationXPDL() {
    final String[] args = new String[0];
    final MainFrame owner = new MainFrame(true, true, args);
    final Document document = new Document();
    final Format format = Format.getPrettyFormat();
    format.setEncoding(XPDLProperty.XPDL_ENCODING);
    final XMLOutputter sortie = new XMLOutputter(format);

    this.xpdlProject = new XPDLProEdProject(this.project, owner);
    this.xpdlProject.toXML(document);
    Assert.assertNotNull(this.xpdlProject);

    // outputString : return a string of all the document
    FileOutputStream fos = null;
    try {
      fos = new FileOutputStream(this.fileName);
      sortie.output(document, fos);

    } catch (final IOException ioe) {
      System.err.println(ioe.getMessage());
    } finally {
      Misc.close(fos);
    }

    document.removeContent();
  }

  public void testLoadXPDL() {
    final String[] args = new String[0];
    final MainFrame owner = new MainFrame(true, true, args);
    final Format format = Format.getPrettyFormat();
    format.setEncoding(XPDLProperty.XPDL_ENCODING);
    final XMLOutputter sortie = new XMLOutputter(format);

    Document doc = null;
    try {
      doc = Utils.loadXMLDocument(this.fileName);
    } catch (final IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    if (doc != null) {
      OutputStream os = null;
      try {
        os = new FileOutputStream(this.fileName);
        sortie.output(doc, os);
      } catch (final IOException ioe) {
        System.err.println(ioe.getMessage());
      } finally {
        Misc.close(os);
      }

      // Load graph model
      final ProEdProject newProject = XPDLProEdProject.toModel(doc, owner);
      Assert.assertNotNull(newProject);
      Assert.assertEquals(this.project, newProject);

    }

  }

  public static Test suite() {
    return new TestSuite(XPDLtest.class);
  }

}